#include "Gadget/Engine.h"
#include "PathNode.h"

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	Core G;
	G.Window.SetSize(640, 640);
	G.Window.SetClearColor(1.0f, 1.0f, 1.0f, 1.0f);
	G.Window.Open();
	G.Mouse.SetVisibility(true);

	glDisable(GL_DEPTH_TEST); //Ignore depth, we'll render everything from back to front

	//It is common in tile based levels for the world to be rendered in patches or groups 
	//of tiles. That way, we only render the patches that are on screen. Rendering a lot of 
	//sprites that are off screen can cause a game to run slow. 
	Image tileImg;
	tileImg.Load("Images/Tiles.png"); //This image contains all the tiles
	Sprite tileSpr(32.0f, 32.0f); //This same sprite will be used for all the tiles

	//Create a patch of 20 by 20 tiles, 400 tiles in all
	const int patchWidth  = 20;
	const int patchHeight = 20;
	//Place the top left corner of the patch 1/2 a tile width from the top and left side
	Vector2D patchPos(16.0f, float(G.Window.GetHeight())-16.0f);

	//Create a map (normally you would load in a file created by a map editor)
	unsigned char patch[patchWidth*patchHeight] = {
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,
		1,0,2,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,
		1,0,0,0,0,0,1,0,0,0,0,0,1,1,1,1,1,0,0,1,
		1,0,0,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,
		1,0,0,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,
		1,0,0,0,0,0,1,0,0,0,0,0,1,1,0,1,1,0,0,1,
		1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,
		1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,1,1,1,1,
		1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,
		1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
		1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,
		1,1,0,0,1,1,0,1,1,1,1,1,1,1,0,0,1,1,1,1,
		1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,
		1,0,0,0,0,0,0,0,0,1,0,0,1,0,1,0,1,0,0,1,
		1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
		1,0,3,0,0,0,0,0,0,1,0,0,1,0,1,0,1,0,0,1,
		1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};

	//Create the nodes
	PathNode node[patchWidth*patchHeight];

	PathNode* goalNode;   //this will point to the node with the gold coin
	PathNode* startNode;  //this will point to the node with the green circle
	PathNode* currentNode;//Used to trace a path from the goal back to the start

	//Loop through the nodes and set their position
	for(int y = 0; y<patchHeight; y++)
	{
		for(int x = 0; x<patchWidth; x++)
		{
			//Set the position of each node
			node[x+y*patchWidth].position.x = patchPos.x + float(x)*tileSpr.width;
			node[x+y*patchWidth].position.y = patchPos.y - float(y)*tileSpr.height;
			node[x+y*patchWidth].patchX = x;
			node[x+y*patchWidth].patchY = y;

			//Check to see if this is the goal node
			if(patch[x+y*patchWidth]==2)
			{
				goalNode = &node[x+y*patchWidth];
				currentNode = goalNode;
			}

			//Check to see if this is the start node
			if(patch[x+y*patchWidth]==3)
			{
				startNode = &node[x+y*patchWidth];
			}
		}
	}

	//Connect the nodes to their neighbours
	for(int y = 0; y<patchHeight; y++)
	{
		for(int x = 0; x<patchWidth; x++)
		{
			//Calculate the distance between this node and the end node
			node[x+y*patchWidth].goalDist = (node[x+y*patchWidth].position-goalNode->position).GetLength();

			//if the node is not on a wall it can have connections to other nodes
			if(patch[x+y*patchWidth]!=1)
			{
				if(patch[x+(y-1)*patchWidth]!=1)//If its not on a wall, connect to node above
				{
					node[x+y*patchWidth].neighbours.push_back(&node[x+(y-1)*patchWidth]);
				}

				if(patch[x+(y+1)*patchWidth]!=1)//If its not on a wall, connect to node below
				{
					node[x+y*patchWidth].neighbours.push_back(&node[x+(y+1)*patchWidth]);
				}

				if(patch[(x-1)+y*patchWidth]!=1)//If its not on a wall, connect to node on left
				{
					node[x+y*patchWidth].neighbours.push_back(&node[(x-1)+y*patchWidth]);
				}

				if(patch[(x+1)+y*patchWidth]!=1)//If its not on a wall, connect to node on right
				{
					node[x+y*patchWidth].neighbours.push_back(&node[(x+1)+y*patchWidth]);
				}

				if(patch[(x-1)+(y-1)*patchWidth]!=1)//If its not on a wall, connect to node left & above
				{
					node[x+y*patchWidth].neighbours.push_back(&node[(x-1)+(y-1)*patchWidth]);
				}

				if(patch[(x+1)+(y-1)*patchWidth]!=1)//If its not on a wall, connect to node right & above
				{
					node[x+y*patchWidth].neighbours.push_back(&node[(x+1)+(y-1)*patchWidth]);
				}

				if(patch[(x-1)+(y+1)*patchWidth]!=1)//If its not on a wall, connect to node left & below
				{
					node[x+y*patchWidth].neighbours.push_back(&node[(x-1)+(y+1)*patchWidth]);
				}

				if(patch[(x+1)+(y+1)*patchWidth]!=1)//If its not on a wall, connect to node right & below
				{
					node[x+y*patchWidth].neighbours.push_back(&node[(x+1)+(y+1)*patchWidth]);
				}
			}
		}
	}

	//The starting node should have a path distance of 0
	startNode->pathDist = 0.0f;

	while(G.Window.IsOpen())//loop as long as the window is open
	{
		G.Renderer.BeginFrame(); G.Renderer.Render2D();

			//ESC closes the window
			if(G.Keyboard.GetKeyHold(GLFW_KEY_ESC)) {G.Window.Close();}	

			//Render the patch
			Shader::Blend(2); //threshold blending
			for(int y = 0; y<patchHeight; y++)
			{
				for(int x = 0; x<patchWidth; x++)
				{
					if(patch[x+y*patchWidth]!=0)//don't render anything if the tile is 0
					{
						//Update the sprite
						tileSpr.x = patchPos.x + float(x)*tileSpr.width;
						tileSpr.y = patchPos.y - float(y)*tileSpr.height;
						tileSpr.SelectAnimationFrame(4, 8, patch[x+y*patchWidth]-1);

						//Render a tile
						tileImg.Activate();
						tileSpr.Render();
					}
				}
			}

			Shader::Deactivate(); //Turns off any shaders or blending

			//Loop through all the nodes and render them
			for(int i = 0; i < patchWidth*patchHeight; i++)
			{
				node[i].Render();
			}

		G.Renderer.EndFrame();	
	}
	return 0;
}			